@@ -46,13 +46,31 @@ long int myatoi( const char *str )
   return ret;
 }
 
-static void * sse_alloc( int len )
+static void * aligned_alloc( int len )
 {
   void *ptr = NULL;
 
+#if defined(HAVE_POSIX_MEMALIGN)
   if( (0 != posix_memalign( (void **)&ptr, 16, len )) || (ptr == NULL) ) {
     throw std::bad_alloc();
   }
+#else
+  // Some platforms will align malloc. Let's try that first.
+  if( ! (ptr = malloc(len)) ) {
+    throw std::bad_alloc();
+  }
+  if( (uintptr_t)ptr & 0xF ) {
+    // The pointer wasn't 16-byte aligned, so try again with valloc
+    free(ptr);
+    if( ! (ptr = valloc(len)) ) {
+      throw std::bad_alloc();
+    }
+    if( (uintptr_t)ptr & 0xF ) {
+      free(ptr);
+      throw std::bad_alloc();
+    }
+  }
+#endif /* !defined(HAVE_POSIX_MEMALIGN) */
 
   return ptr;
 }
@@ -113,7 +131,7 @@ string Base64Key::printable_key( void ) const
 }
 
 Session::Session( Base64Key s_key )
-  : key( s_key ), ctx( NULL )
+  : key( s_key ), ctx( NULL ), blocks_encrypted( 0 )
 {
   ctx = ae_allocate( NULL );
   if ( ctx == NULL ) {
@@ -175,8 +193,8 @@ string Session::encrypt( Message plaintext )
   const size_t pt_len = plaintext.text.size();
   const int ciphertext_len = pt_len + 16;
 
-  char *ciphertext = (char *)sse_alloc( ciphertext_len );
-  char *pt = (char *)sse_alloc( pt_len );
+  char *ciphertext = (char *)aligned_alloc( ciphertext_len );
+  char *pt = (char *)aligned_alloc( pt_len );
 
   memcpy( pt, plaintext.text.data(), plaintext.text.size() );
 
@@ -198,6 +216,30 @@ string Session::encrypt( Message plaintext )
     throw CryptoException( "ae_encrypt() returned error." );
   }
 
+  blocks_encrypted += pt_len >> 4;
+  if ( pt_len & 0xF ) {
+    /* partial block */
+    blocks_encrypted++;
+  }
+
+  /* "Both the privacy and the authenticity properties of OCB degrade as
+      per s^2 / 2^128, where s is the total number of blocks that the
+      adversary acquires.... In order to ensure that s^2 / 2^128 remains
+      small, a given key should be used to encrypt at most 2^48 blocks (2^55
+      bits or 4 petabytes)"
+
+     -- http://tools.ietf.org/html/draft-krovetz-ocb-03
+
+     We deem it unlikely that a legitimate user will send 4 PB through a Mosh
+     session.  If it happens, we simply kill the session.  The server and
+     client use the same key, so we actually need to die after 2^47 blocks.
+  */
+  if ( blocks_encrypted >> 47 ) {
+    free( pt );
+    free( ciphertext );
+    throw CryptoException( "Encrypted 2^47 blocks.", true );
+  }
+
   string text( (char *)ciphertext, ciphertext_len );
   free( pt );
   free( ciphertext );
@@ -222,10 +264,10 @@ Message Session::decrypt( string ciphertext )
   }
 
   Nonce __attribute__((__aligned__ (16))) nonce( str, 8 );
-  char *body = (char *)sse_alloc( body_len );
+  char *body = (char *)aligned_alloc( body_len );
   memcpy( body, str + 8, body_len );
 
-  char *plaintext = (char *)sse_alloc( pt_len );
+  char *plaintext = (char *)aligned_alloc( pt_len );
 
   if ( pt_len != ae_decrypt( ctx,               /* ctx */
 			     nonce.data(),      /* nonce */
