@@ -23,10 +23,9 @@
 #include <stdlib.h>
 #include <signal.h>
 #include <errno.h>
-#include <poll.h>
+#include <sys/poll.h>
 #include <string.h>
 #include <locale.h>
-#include <langinfo.h>
 #include <wchar.h>
 #include <assert.h>
 #include <wctype.h>
@@ -50,6 +49,7 @@
 #include "completeterminal.h"
 #include "swrite.h"
 #include "fatal_assert.h"
+#include "locale_utils.h"
 
 extern "C" {
 #include "selfpipe.h"
@@ -64,15 +64,8 @@ int main( void )
   int master;
   struct termios saved_termios, raw_termios, child_termios;
 
-  if ( NULL == setlocale( LC_ALL, "" ) ) {
-    perror( "setlocale" );
-    exit( 1 );
-  }
-
-  if ( strcmp( nl_langinfo( CODESET ), "UTF-8" ) != 0 ) {
-    fprintf( stderr, "stm requires a UTF-8 locale.\n" );
-    exit( 1 );
-  }
+  set_native_locale();
+  assert_utf8_locale();
 
   if ( tcgetattr( STDIN_FILENO, &saved_termios ) < 0 ) {
     perror( "tcgetattr" );
@@ -81,10 +74,14 @@ int main( void )
 
   child_termios = saved_termios;
 
+#ifdef HAVE_IUTF8
   if ( !(child_termios.c_iflag & IUTF8) ) {
     fprintf( stderr, "Warning: Locale is UTF-8 but termios IUTF8 flag not set. Setting IUTF8 flag.\n" );
     child_termios.c_iflag |= IUTF8;
   }
+#else
+  fprintf( stderr, "Warning: termios IUTF8 flag not defined. Character-erase of multibyte character sequence probably does not work properly on this platform.\n" );
+#endif /* HAVE_IUTF8 */
 
   pid_t child = forkpty( &master, NULL, &child_termios, NULL );
 
