@@ -21,12 +21,11 @@
 #include <errno.h>
 #include <locale.h>
 #include <string.h>
-#include <langinfo.h>
 #include <termios.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <poll.h>
+#include <sys/poll.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <pwd.h>
@@ -49,6 +48,7 @@ extern "C" {
 #include "swrite.h"
 #include "user.h"
 #include "fatal_assert.h"
+#include "locale_utils.h"
 
 #if HAVE_PTY_H
 #include <pty.h>
@@ -196,16 +196,8 @@ int main( int argc, char *argv[] )
   }
 
   /* Adopt implementation locale */
-  if ( NULL == setlocale( LC_ALL, "" ) ) {
-    perror( "setlocale" );
-    exit( 1 );
-  }
-
-  /* Verify locale calls for UTF-8 */
-  if ( strcmp( nl_langinfo( CODESET ), "UTF-8" ) != 0 ) {
-    fprintf( stderr, "mosh requires a UTF-8 locale.\n" );
-    exit( 1 );
-  }
+  set_native_locale();
+  assert_utf8_locale();
 
   try {
     return run_server( desired_ip, desired_port, command, colors );
@@ -273,11 +265,15 @@ int run_server( const char *desired_ip, const char *desired_port,
 
   int master;
 
+#ifdef HAVE_IUTF8
   if ( !(child_termios.c_iflag & IUTF8) ) {
     /* SSH should also convey IUTF8 across connection. */
     //    fprintf( stderr, "Warning: Locale is UTF-8 but termios IUTF8 flag not set. Setting IUTF8 flag.\n" );
     child_termios.c_iflag |= IUTF8;
   }
+#else
+  fprintf( stderr, "\nWarning: termios IUTF8 flag not defined.\nCharacter-erase of multibyte character sequence\nprobably does not work properly on this platform.\n" );
+#endif /* HAVE_IUTF8 */
 
   /* Fork child process */
   pid_t child = forkpty( &master, NULL, &child_termios, &window_size );
@@ -437,6 +433,10 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
 	      /* tell child process of resize */
 	      const Parser::Resize *res = static_cast<const Parser::Resize *>( us.get_action( i ) );
 	      struct winsize window_size;
+	      if ( ioctl( host_fd, TIOCGWINSZ, &window_size ) < 0 ) {
+		perror( "ioctl TIOCGWINSZ" );
+		return;
+	      }
 	      window_size.ws_col = res->width;
 	      window_size.ws_row = res->height;
 	      if ( ioctl( host_fd, TIOCSWINSZ, &window_size ) < 0 ) {
@@ -592,7 +592,11 @@ void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &network
       fprintf( stderr, "%s: %s\n", e.function.c_str(), strerror( e.the_errno ) );
       spin();
     } catch ( Crypto::CryptoException e ) {
-      fprintf( stderr, "Crypto exception: %s\n", e.text.c_str() );
+      if ( e.fatal ) {
+        throw;
+      } else {
+        fprintf( stderr, "Crypto exception: %s\n", e.text.c_str() );
+      }
     }
   }
 }
