@@ -21,11 +21,10 @@
 #include <errno.h>
 #include <locale.h>
 #include <string.h>
-#include <langinfo.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <poll.h>
+#include <sys/poll.h>
 #include <sys/ioctl.h>
 #include <sys/types.h>
 #include <pwd.h>
@@ -47,16 +46,13 @@ extern "C" {
 #include "completeterminal.h"
 #include "user.h"
 #include "fatal_assert.h"
+#include "locale_utils.h"
 
 #include "networktransport.cc"
 
 void STMClient::init( void )
 {
-  /* Verify locale calls for UTF-8 */
-  if ( strcmp( nl_langinfo( CODESET ), "UTF-8" ) != 0 ) {
-    fprintf( stderr, "mosh requires a UTF-8 locale.\n" );
-    exit( 1 );
-  }
+  assert_utf8_locale();
 
   /* Verify terminal configuration */
   if ( tcgetattr( STDIN_FILENO, &saved_termios ) < 0 ) {
@@ -66,11 +62,14 @@ void STMClient::init( void )
 
   /* Put terminal driver in raw mode */
   raw_termios = saved_termios;
+
+#ifdef HAVE_IUTF8
   if ( !(raw_termios.c_iflag & IUTF8) ) {
-    /* SSH should also convey IUTF8 across connection. */
     //    fprintf( stderr, "Warning: Locale is UTF-8 but termios IUTF8 flag not set. Setting IUTF8 flag.\n" );
+    /* Probably not really necessary since we are putting terminal driver into raw mode anyway. */
     raw_termios.c_iflag |= IUTF8;
   }
+#endif /* HAVE_IUTF8 */
 
   cfmakeraw( &raw_termios );
 
@@ -399,9 +398,13 @@ void STMClient::main( void )
       req.tv_nsec = 200000000; /* 0.2 sec */
       nanosleep( &req, NULL );
     } catch ( Crypto::CryptoException e ) {
-      wchar_t tmp[ 128 ];
-      swprintf( tmp, 128, L"Crypto exception: %s", e.text.c_str() );
-      overlays.get_notification_engine().set_notification_string( wstring( tmp ) );
+      if ( e.fatal ) {
+        throw;
+      } else {
+        wchar_t tmp[ 128 ];
+        swprintf( tmp, 128, L"Crypto exception: %s", e.text.c_str() );
+        overlays.get_notification_engine().set_notification_string( wstring( tmp ) );
+      }
     }
   }
 }
