@@ -39,7 +39,7 @@ private:
   int signal_fd;
   struct winsize window_size;
 
-  Terminal::Framebuffer *local_framebuffer;
+  Terminal::Framebuffer *local_framebuffer, *new_state;
   Overlay::OverlayManager overlays;
   Network::Transport< Network::UserStream, Terminal::Complete > *network;
   Terminal::Display display;
@@ -53,6 +53,8 @@ private:
 
   void output_new_frame( void );
 
+  bool still_connecting( void ) { return (network->get_remote_state_num() == 0); }
+
 public:
   STMClient( const char *s_ip, int s_port, const char *s_key, const char *predict_mode )
     : ip( s_ip ), port( s_port ), key( s_key ),
@@ -60,6 +62,7 @@ public:
       signal_fd(),
       window_size(),
       local_framebuffer( NULL ),
+      new_state( NULL ),
       overlays(),
       network( NULL ),
       display( true ), /* use TERM environment var to initialize display */
@@ -90,6 +93,10 @@ public:
       delete local_framebuffer;
     }
 
+    if ( new_state != NULL ) {
+      delete new_state;
+    }
+
     if ( network != NULL ) {
       delete network;
     }
