@@ -27,6 +27,7 @@
 #include "transportinstruction.pb.h"
 #include "transportstate.h"
 #include "transportfragment.h"
+#include "prng.h"
 
 using std::list;
 using std::pair;
@@ -86,6 +87,10 @@ namespace Network {
 
     uint64_t last_heard; /* last time received new state */
 
+    /* chaff to disguise instruction length */
+    PRNG prng;
+    const string make_chaff( void );
+
   public:
     /* constructor */
     TransportSender( Connection *s_connection, MyState &initial_state );
